/*
 * This file is part of Bipscript-IDE.
 *
 * Bipscript-IDE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Bipscript-IDE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Bipscript-IDE.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PROJECTLIST_H
#define PROJECTLIST_H

#include "namedfile.h"

#include <QList>
#include <QSet>
#include <QString>

class Project {
    QList<NamedFile> folders;
    QSet<QString> mopenFiles;
    QString mexecutable;
    QString mactiveFile;

  public:
    Project();
    int folderCount() { return folders.size(); }
    NamedFile &folder(int index) { return folders[index]; }
    void addFolder(NamedFile folder) { folders.append(folder); }
    void openFile(const QString &path) { mopenFiles.insert(path); }
    void closeFile(const QString &path) { mopenFiles.remove(path); }
    QSet<QString> openFiles() { return mopenFiles; }
    QString &executable() { return mexecutable; }
    void executable(const QString &exec) { mexecutable = exec; }
    QString &activeFile() { return mactiveFile; }
    void activeFile(const QString &active) { mactiveFile = active; }
    bool exists();
};

class ProjectList {
    QList<Project> projects;
    int openIndex;

  private:
    ProjectList() {}
    ProjectList(const ProjectList &) {}
    QString commonRoot(QString &path1, QString &path2);

  public:
    static ProjectList &instance() {
        static ProjectList *instance = nullptr;
        if (instance == nullptr) {
            instance = new ProjectList();
        }
        return *instance;
    }
    // persistence
    void load();
    void save();
    // for main window
    bool currentExists();
    Project &currentProject() { return projects[openIndex]; }
    // for project management dialog
    int size() { return projects.size(); }
    Project &get(int index) { return projects[index]; }
    QString commonRoot();
    int findProject(QString &path);
    void addProject(Project &project) { projects.append(project); }
    void removeProject(int index);
    bool isOpen(int index) { return index == openIndex; }
    bool open(int index);
    bool select(QString path);
};
#endif // PROJECTLIST_H
